@@ -277,6 +277,7 @@ our %EXTRAS;
 sub pdlpp_eumm_update_deep {
   my ($eumm) = @_;
   my $pm = $eumm->{PM};
+  delete @$pm{grep /\Q$Config{obj_ext}\E$/, keys %$pm};
   my $macro = $eumm->{macro} ||= {};
   my $xsb = $eumm->{XSBUILD}{xs} ||= {};
   $eumm->{clean}{FILES} ||= '';
@@ -662,6 +663,7 @@ prints on C<STDOUT> XS text with core flags, for F<Core.xs>.
 my %flags = (
     hdrcpy => { set => 1 },
     fflows => { FLAG => "DATAFLOW_F" },
+    is_readonly => { FLAG => "READONLY" },
     is_inplace => { FLAG => "INPLACE", postset => 1 },
     set_inplace => { FLAG => "INPLACE", noret => 1 },
     donttouch => { FLAG => "DONTTOUCHDATA" },
