@@ -49,7 +49,6 @@ our @EXPORT = qw( apropos aproposover usage help sig badinfo whatis );
 use PDL::Doc;
 use Pod::Select;
 use Pod::PlainText;
-use Term::ReadKey; #for GetTerminalSize
 use Cwd; # to help Debian packaging
 
 $PDL::onlinedoc = PDL::Doc->new(FindStdFile());
@@ -73,8 +72,12 @@ sub FindStdFile {
 # to work (at least on my solaris and linux
 # machines)
 #
-sub screen_width() {
-    return ( ( GetTerminalSize(\*STDOUT) )[0] // 72);
+sub screen_width {
+  local $@;
+  eval {
+    require Term::ReadKey;
+    ( Term::ReadKey::GetTerminalSize(\*STDOUT) )[0];
+  } // 72;
 }
 
 sub printmatch {
@@ -179,12 +182,11 @@ with the C<help> function
 =cut
 
 sub aproposover {
-    die "Usage: aproposover \$funcname\n" unless $#_>-1;
+    die "Usage: aproposover \$funcname\n" if !@_;
     die "no online doc database" unless defined $PDL::onlinedoc;
     my $func = shift;
     $func =~ s:\/:\\\/:g;
     search_docs("m/$func/",['Name','Ref','Module'],1);
-
 }
 
 sub apropos  {
@@ -617,7 +619,7 @@ and the remaining commands listed, along with the names of their modules.
 =cut
 
 sub help {
-  if ($#_>-1) {
+  if (@_) {
       require PDL::Dbg;
       my $topic = shift;
       if (PDL::Core::blessed($topic) && $topic->can('px')) {
@@ -655,6 +657,7 @@ Quick start:
 
 EOH
   }
+  ''
 }
 
 =head2 badinfo
