@@ -4,7 +4,8 @@
 # This is what makes the nice loops go around etc.
 #
 
-package PDL::PP::Code;
+package # hide from PAUSE/MetaCPAN
+  PDL::PP::Code;
 
 use strict;
 use warnings;
@@ -69,6 +70,7 @@ sub new {
         Generictypes => $generictypes,   # so that MacroAccess can check it
         Name => $name,
         NullDataCheck => $nulldatacheck,
+        BadFlag => $handlebad,
     }, $class;
 
     my @codes = $code;
@@ -156,7 +158,7 @@ sub params_declare {
     my ($this) = @_;
     my ($ord,$pdls) = $this->get_pdls;
     my %istyped = map +($_=>1), grep $pdls->{$_}{FlagTypeOverride}, @$ord;
-    my @decls = map $_->get_xsdatapdecl($istyped{$_->name} ? "PDL_TYPE_PARAM_".$_->name : "PDL_TYPE_OP", $this->{NullDataCheck}, $istyped{$_->name} ? "PDL_PPSYM_PARAM_".$_->name : "PDL_PPSYM_OP"),
+    my @decls = map $_->get_xsdatapdecl($istyped{$_->name} ? "PDL_TYPE_PARAM_".$_->name : "PDL_TYPE_OP", $this->{NullDataCheck}, $istyped{$_->name} ? "PDL_PPSYM_PARAM_".$_->name : "PDL_PPSYM_OP", $this->{BadFlag}),
       map $pdls->{$_}, @$ord;
     my @param_names = ("PDL_TYPE_OP", "PDL_PPSYM_OP", map +("PDL_TYPE_PARAM_$_","PDL_PPSYM_PARAM_$_"), grep $istyped{$_}, @$ord);
     <<EOF;
@@ -348,7 +350,8 @@ sub report_error {
 # 	new - constructor
 #	get_str - get the string to be put into the xsub.
 
-package PDL::PP::Block;
+package # hide from PAUSE/MetaCPAN
+  PDL::PP::Block;
 
 sub new { my($type) = @_; bless [],$type; }
 
@@ -398,7 +401,8 @@ sub enter {
 # - ie create something like
 #   if ( badflag ) { badcode } else { goodcode }
 #
-package PDL::PP::BadSwitch;
+package # hide from PAUSE/MetaCPAN
+  PDL::PP::BadSwitch;
 our @ISA = "PDL::PP::Block";
 
 sub new {
@@ -428,7 +432,8 @@ if ( \$PRIV(bvalflag) ) { /* ** do 'bad' Code ** */
 EOF
 }
 
-package PDL::PP::Loop;
+package # hide from PAUSE/MetaCPAN
+  PDL::PP::Loop;
 our @ISA = "PDL::PP::Block";
 
 sub new { my($type,$args,$sizeprivs,$parent) = @_;
@@ -479,7 +484,8 @@ sub mypostlude { my($this,$parent,$context) = @_;
   return join '', map "}} /* Close $_ */", @{$this->[0]};
 }
 
-package PDL::PP::GenericSwitch;
+package # hide from PAUSE/MetaCPAN
+  PDL::PP::GenericSwitch;
 use Carp;
 our @ISA = "PDL::PP::Block";
 
@@ -526,7 +532,7 @@ sub myitemstart {
       grep $istyped{$_}, @$ord);
     my $decls = keys %{$this->[2]} == @$ord
       ? "PDL_DECLARE_PARAMS_$parent->{Name}_$parent->{NullDataCheck}(@{[join ',', @param_ctypes]})\n"
-      : join '', map $_->get_xsdatapdecl($_->adjusted_type($item)->ctype, $parent->{NullDataCheck}, $_->adjusted_type($item)->ppsym),
+      : join '', map $_->get_xsdatapdecl($_->adjusted_type($item)->ctype, $parent->{NullDataCheck}, $_->adjusted_type($item)->ppsym, $parent->{BadFlag}),
           map $parent->{ParObjs}{$_}, sort keys %{$this->[2]};
     my @gentype_decls = !$this->[4] ? () : map "#define PDL_IF_GENTYPE_".uc($_)."(t,f) ".
 	($item->$_ ? 't' : 'f')."\n",
@@ -559,7 +565,8 @@ sub mypostlude {
 # This relies on PP.pm making sure that initbroadcaststruct always sets
 # up the two first dimensions even when they are not necessary.
 #
-package PDL::PP::BroadcastLoop;
+package # hide from PAUSE/MetaCPAN
+  PDL::PP::BroadcastLoop;
 use Carp;
 our @ISA = "PDL::PP::Block";
 
@@ -580,7 +587,8 @@ sub mypostlude {my($this,$parent,$context,$backcode) = @_;
 # Simple subclass of BroadcastLoop to implement writeback code
 #
 #
-package PDL::PP::BackCodeBroadcastLoop;
+package # hide from PAUSE/MetaCPAN
+  PDL::PP::BackCodeBroadcastLoop;
 use Carp;
 our @ISA = "PDL::PP::BroadcastLoop";
 
@@ -602,7 +610,8 @@ sub mypostlude {
 #
 # Encapsulate a types() switch
 #
-package PDL::PP::Types;
+package # hide from PAUSE/MetaCPAN
+  PDL::PP::Types;
 use Carp;
 use PDL::Types ':All';
 our @ISA = "PDL::PP::Block";
@@ -624,7 +633,8 @@ sub get_str {
 }
 
 
-package PDL::PP::Access;
+package # hide from PAUSE/MetaCPAN
+  PDL::PP::Access;
 use Carp;
 
 sub new { my($type,$pdl,$inds) = @_;
@@ -639,7 +649,8 @@ sub get_str { my($this,$parent,$context) = @_;
 ###########################
 # Encapsulate a check on whether a value is good or bad
 # handles both checking (good/bad) and setting (bad)
-package PDL::PP::BadAccess;
+package # hide from PAUSE/MetaCPAN
+  PDL::PP::BadAccess;
 use Carp;
 
 sub new {
@@ -682,7 +693,8 @@ sub get_str {
 }
 
 
-package PDL::PP::MacroAccess;
+package # hide from PAUSE/MetaCPAN
+  PDL::PP::MacroAccess;
 use Carp;
 use PDL::Types ':All';
 my $types = join '',ppdefs_all;
@@ -712,7 +724,8 @@ sub get_str {
     $type2value->{$parent->{Gencurtype}[-1]->ppsym};
 }
 
-package PDL::PP::GentypeAccess;
+package # hide from PAUSE/MetaCPAN
+  PDL::PP::GentypeAccess;
 use Carp;
 
 sub new { my($type,$pdl,$inds) = @_; bless [$inds],$type; }
@@ -725,7 +738,8 @@ sub get_str {my($this,$parent,$context) = @_;
   $pobj->adjusted_type($type)->ctype;
 }
 
-package PDL::PP::PpsymAccess;
+package # hide from PAUSE/MetaCPAN
+  PDL::PP::PpsymAccess;
 use Carp;
 
 sub new { my($type,$pdl,$inds) = @_; bless [$inds],$type; }
